#include"krtdb_stream.hpp"




namespace{


uint32_t
crc_table[256];


void
make_crc_table()
{
    for(uint32_t  i = 0;  i < 256;  i++)
    {
      uint32_t c = i;

        for(int  j = 0;  j < 8;  j++)
        {
          c = (c & 1) ? (0xEDB88320 ^ (c >> 1)) : (c >> 1);
        }


      crc_table[i] = c;
    }
}


}




namespace krtdb{




Stream::
Stream():
crc32_flag_(0),
taskname_("")
{
}


Stream::
~Stream()
{
}




size_t
Stream::
get_pos() const
{
  return 0;
}


void
Stream::
set_pos(size_t  pos)
{
}


int
Stream::
get()
{
  return 0;
}


int
Stream::
put(int  c)
{
  auto  uc = static_cast<unsigned char>(c);

    if(crc32_flag_)
    {
      crc32_ = crc_table[(crc32_ ^ uc) & 0xFF] ^ (crc32_ >> 8);
    }
	

  return uc;
}




int8_t
Stream::
get_int8()
{
  return get();
}


uint8_t
Stream::
get_uint8()
{
  return get();
}


int16_t
Stream::
get_int16_be()
{
  return get_uint16_be();
}


uint16_t
Stream::
get_uint16_be()
{
  uint16_t  u = 0;

  u |= (get()<<8);
  u |= (get()   );

  return u;
}


int32_t
Stream::
get_int32_be()
{
  return get_uint32_be();
}


uint32_t
Stream::
get_uint32_be()
{
  uint32_t  u = 0;

  u |= (get()<<24);
  u |= (get()<<16);
  u |= (get()<< 8);
  u |= (get()    );

  return u;
}


int64_t
Stream::
get_int64_be()
{
  return get_uint64_be();
}


uint64_t
Stream::
get_uint64_be()
{
  uint64_t  u = 0;

  u |= (static_cast<uint64_t>(get())<<56);
  u |= (static_cast<uint64_t>(get())<<48);
  u |= (static_cast<uint64_t>(get())<<40);
  u |= (static_cast<uint64_t>(get())<<32);
  u |= (static_cast<uint64_t>(get())<<24);
  u |= (static_cast<uint64_t>(get())<<16);
  u |= (static_cast<uint64_t>(get())<< 8);
  u |= (static_cast<uint64_t>(get())    );

  return u;
}




int16_t
Stream::
put_int16_be( int16_t  c)
{
  return put_uint16_be(c);
}


uint16_t
Stream::
put_uint16_be(uint16_t  c)
{
  put((c>>8)&0xFF);
  put((c   )&0xFF);

  return c;
}


int32_t
Stream::
put_int32_be( int32_t  c)
{
  return put_uint32_be(c);
}


uint32_t
Stream::
put_uint32_be(uint32_t  c)
{
  put((c>>24)&0xFF);
  put((c>>16)&0xFF);
  put((c>> 8)&0xFF);
  put((c    )&0xFF);

  return c;
}


int64_t
Stream::
put_int64_be( int64_t  c)
{
  return put_uint64_be(c);
}


uint64_t
Stream::
put_uint64_be(uint64_t  c)
{
  put((c>>56)&0xFF);
  put((c>>48)&0xFF);
  put((c>>40)&0xFF);
  put((c>>32)&0xFF);
  put((c>>24)&0xFF);
  put((c>>16)&0xFF);
  put((c>> 8)&0xFF);
  put((c    )&0xFF);

  return c;
}





void
Stream::
reset_crc32(bool  v)
{
  static bool  initialized;

    if(!initialized)
    {
      make_crc_table();

      initialized = true;
    }


  crc32_ = 0xFFFFFFFF;

  crc32_flag_ = v;
}


uint32_t
Stream::
get_crc32() const
{
  return crc32_ ^ 0xFFFFFFFF;
}




void
Stream::
set_taskname(const char*  s)
{
  taskname_ = s;
}


void
Stream::
write(const void*  buf, size_t  bytes)
{
  auto  src = static_cast<const char*>(buf);

    while(bytes--)
    {
      put(*src++);
    }
}


void
Stream::
read(void*  buf, size_t  bytes)
{
  auto  dst = static_cast<char*>(buf);

    while(bytes--)
    {
      *dst++ = get();
    }
}


void
Stream::
print() const
{
  printf("[stream print] %s\n",taskname_);
}




MemoryStream::
MemoryStream():
unique_s_(new std::string),
s_(unique_s_.get()),
offset_(0)
{
}


MemoryStream::
MemoryStream(std::string&  s):
s_(&s),
offset_(0)
{
}




const std::string&
MemoryStream::
get_buffer() const
{
  return *s_;
}


ByteString
MemoryStream::
get_byte_string() const
{
  ByteString  bs;

  auto&  buf = get_buffer();

  bs.create_buffer(buf.size(),buf.data());

  return bs;
}


size_t
MemoryStream::
get_pos() const
{
  return offset_;
}


void
MemoryStream::
set_pos(size_t  pos)
{
  offset_ = pos;
}


int
MemoryStream::
get()
{
  return static_cast<uint8_t>((*s_)[offset_++]);
}


int
MemoryStream::
put(int  c)
{
  c = Stream::put(c);

    if(offset_ < s_->size())
    {
      (*s_)[offset_] = c;
    }

  else
    {
      s_->push_back(c);
    }


  offset_ += 1;

  return c;
}




ConstMemoryStream::
ConstMemoryStream(const void*  buffer, size_t  size):
buffer_(static_cast<const uint8_t*>(buffer)),
size_(size),
offset_(0)
{
}




void
ConstMemoryStream::
set_pos(size_t  pos)
{
  offset_ = pos;
}


size_t
ConstMemoryStream::
get_pos() const
{
  return offset_;
}


int
ConstMemoryStream::
get()
{
  return buffer_[offset_++];
}




FileStream::
FileStream(FILE*  f): f_(f)
{
}


FileStream::
~FileStream()
{
    if(f_)
    {
      fclose(f_);
    }
}




void
FileStream::
set_pos(size_t  pos)
{
  fseek(f_,pos,SEEK_SET);
}


size_t
FileStream::
get_pos() const
{
  return ftell(f_);
}


int
FileStream::
get()
{
  return fgetc(f_);
}


int
FileStream::
put(int  c)
{
  Stream::put(c);

  return fputc(c,f_);
}




}




