#ifndef KRTDB_STREAM_HPP_INCLUDED
#define KRTDB_STREAM_HPP_INCLUDED


#include<cstdio>
#include<cstdint>
#include<cstddef>
#include<cstring>
#include<string>
#include<memory>
#include<utility>
#include<cinttypes>
#include<limits>
#include"krtdb_bytestring.hpp"




namespace krtdb{


using  vlint_t =  int32_t;
using uvlint_t = uint32_t;


size_t  get_vlint_size(uvlint_t  i);


class
Stream
{
  uint32_t  crc32_;

  int  crc32_flag_;

  const char*  taskname_;

public:
           Stream();
  virtual ~Stream();

  virtual size_t  get_pos(           ) const;
  virtual void    set_pos(size_t  pos)      ;

  virtual int  get(      );
  virtual int  put(int  c);

   int8_t   get_int8();
  uint8_t  get_uint8();

   int16_t   get_int16_be();
  uint16_t  get_uint16_be();
   int32_t   get_int32_be();
  uint32_t  get_uint32_be();
   int64_t   get_int64_be();
  uint64_t  get_uint64_be();

   vlint_t   get_vlint();
  uvlint_t  get_uvlint();

   int16_t   put_int16_be( int16_t  c);
  uint16_t  put_uint16_be(uint16_t  c);
   int32_t   put_int32_be( int32_t  c);
  uint32_t  put_uint32_be(uint32_t  c);
   int64_t   put_int64_be( int64_t  c);
  uint64_t  put_uint64_be(uint64_t  c);

  void   put_vlint( vlint_t  c);
  void  put_uvlint(uvlint_t  c);

  void  set_taskname(const char*  s="");

  void  reset_crc32(bool  v);

  uint32_t  get_crc32() const;

  void  write(const void*  buf, size_t  bytes);
  void   read(      void*  buf, size_t  bytes);

  void  print() const;

};




class
MemoryStream: public Stream
{
  std::unique_ptr<std::string>  unique_s_;

  std::string*  s_;

  size_t  offset_;

public:
   MemoryStream(               );
   MemoryStream(std::string&  s);


  const std::string&  get_buffer() const;

  ByteString  get_byte_string() const;

  size_t  get_pos(           ) const;
  void    set_pos(size_t  pos)      ;

  int  get();
  int  put(int  c);

};


class
ConstMemoryStream: public Stream
{
  const uint8_t*  buffer_;

  size_t  size_;

  size_t  offset_;

public:
   ConstMemoryStream(const void*  buffer, size_t  size);

  size_t  get_pos(           ) const;
  void    set_pos(size_t  pos)      ;

  int  get();

};




class
FileStream: public Stream
{
  FILE*  f_;

public:
   FileStream(FILE*  f);
  ~FileStream();

  size_t  get_pos(           ) const;
  void    set_pos(size_t  pos)      ;

  int  get(      );
  int  put(int  c);

};




}




#endif




