#include"krtdb_stream.hpp"




namespace krtdb{


/*
1byte ,  7bits 1xxxxxxx

2bytes, 12bits 0LLLxxxx xxxxxxxx
3bytes, 20bits 0LLLxxxx xxxxxxxx xxxxxxxx
4bytes, 28bits 0LLLxxxx xxxxxxxx xxxxxxxx xxxxxxxx
5bytes, 36bits 0LLL0000 xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx

//6bytes, 44bits 0LLLxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx
//7bytes, 52bits 0LLLxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx
//8bytes, 60bits 0LLLxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx
//9bytes, 64bits 0LLL0000 xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx


Lの部分3ビットが表す値に2を足したものが、可変長整数のバイト数
*/


size_t
get_vlint_size(uvlint_t  i)
{
  size_t  size;

       if(i <= 0x0000007F){size = 1;}
  else if(i <= 0x00000FFF){size = 2;}
  else if(i <= 0x000FFFFF){size = 3;}
  else if(i <= 0x0FFFFFFF){size = 4;}
  else if(i <= 0xFFFFFFFF){size = 5;}

  return size;
}


void
Stream::
put_vlint(vlint_t  i)
{
  const bool  negative = (i < 0);

  i = std::abs(i)<<1;

    if(negative)
    {
      i |= 1;
    }


  put_uvlint(i);
}


void
Stream::
put_uvlint(uvlint_t  i)
{
    if(i <= 0x7F)
    {
      put(i|0x80);

      return;
    }


    switch(get_vlint_size(i))
    {
      case(2):
        put(i>>8);
        put(i&0xFF);
        break;
      case(3):
        put((1<<4)|(i>>16));
        put((i>>8)&0xFF);
        put((i>>0)&0xFF);
        break;
      case(4):
        put((2<<4)|(i>>24));
        put((i>>16)&0xFF);
        put((i>> 8)&0xFF);
        put((i>> 0)&0xFF);
        break;
      case(5):
        put(3<<4);
        put((i>>24)&0xFF);
        put((i>>16)&0xFF);
        put((i>> 8)&0xFF);
        put((i>> 0)&0xFF);
        break;
    }
}


vlint_t
Stream::
get_vlint()
{
  auto i = get_uvlint();

  const bool negative = (i&1);

  i >>= 1;

  return(negative? -i:i);
}


uvlint_t
Stream::
get_uvlint()
{
  uint8_t  first_byte = get();

    if(first_byte&0x80)
    {
      return(first_byte&0x7F);
    }


  size_t  size = (first_byte>>4);

  uint32_t  i;

    switch(size)
    {
      case(0):
        i = ((first_byte&0x0F)<<8);
        i |= get();
        break;
      case(1):
        i = ((first_byte&0x0F)<<16);
        i |= (get()<<8);
        i |= (get()   );
        break;
      case(2):
        i = ((first_byte&0x0F)<<24);
        i |= (get()<<16);
        i |= (get()<< 8);
        i |= (get()    );
        break;
      case(3):
        i = 0;//((first_byte&0x0F)<<32);
        i |= (get()<<24);
        i |= (get()<<16);
        i |= (get()<< 8);
        i |= (get()    );
        break;
    }


  return i;
}


}




