#include"krtdb.hpp"
#include<random>




using namespace krtdb;




template<typename  INT>
void
print_int(INT  v)
{
    if(std::numeric_limits<INT>::is_signed)
    {
      printf("%lld\r",static_cast<int64_t>(v));
    }

  else
    {
      printf("%llu\r",static_cast<uint64_t>(v));
    }
}


template<typename  INT, typename  PUT, typename  GET>
bool
test_stream_int_rw(MemoryStream&  m, INT  v, PUT  put_fn, GET  get_fn)
{
  m.set_pos(0);

  (m.*put_fn)(v);

//  auto  w_size = m.get_pos();

  m.set_pos(0);

  auto  res = (m.*get_fn)();

//  auto  r_size = m.get_pos();

  return(v == res);
}


template<typename  INT, typename  PUT, typename  GET>
bool
test_stream_random_int_rw(MemoryStream&  m, bool  vl, PUT  put_fn, GET  get_fn)
{
  printf("[%ssigned %dbit %sinteger test]\n",std::numeric_limits<INT>::is_signed? "  ":"un",sizeof(INT)*8,vl? "variable-length-":"");


  std::random_device  seed;

  std::default_random_engine  engine(seed());

  std::uniform_int_distribution<INT>  dist(std::numeric_limits<INT>::min(),
                                           std::numeric_limits<INT>::max());

    for(int  i = 128;  i;  --i)
    {
      const auto  v = dist(engine);

      print_int(v);

      fflush(stdout);

        if(!test_stream_int_rw(m,v,put_fn,get_fn))
        {
          printf("failed.\n");

          return false;
        }
    }


  return true;
}




void
test_vlint()
{
  MemoryStream  m;

    if(!test_stream_random_int_rw< int16_t>(m,false,&Stream::put_int16_be,&Stream::get_int16_be  ) ||
       !test_stream_random_int_rw<uint16_t>(m,false,&Stream::put_uint16_be,&Stream::get_uint16_be) ||
       !test_stream_random_int_rw< int32_t>(m,false,&Stream::put_int32_be,&Stream::get_int32_be  ) ||
       !test_stream_random_int_rw<uint32_t>(m,false,&Stream::put_uint32_be,&Stream::get_uint32_be) ||
       !test_stream_random_int_rw< int64_t>(m,false,&Stream::put_int64_be,&Stream::get_int64_be  ) ||
       !test_stream_random_int_rw<uint64_t>(m,false,&Stream::put_uint64_be,&Stream::get_uint64_be) ||
       !test_stream_random_int_rw<  int8_t>(m, true,&Stream::put_vlint,&Stream::get_vlint        ) ||
       !test_stream_random_int_rw< uint8_t>(m, true,&Stream::put_uvlint,&Stream::get_uvlint      ) ||
       !test_stream_random_int_rw< int16_t>(m, true,&Stream::put_vlint,&Stream::get_vlint        ) ||
       !test_stream_random_int_rw<uint16_t>(m, true,&Stream::put_uvlint,&Stream::get_uvlint      ) ||
       !test_stream_random_int_rw< int32_t>(m, true,&Stream::put_vlint,&Stream::get_vlint        ) ||
       !test_stream_random_int_rw<uint32_t>(m, true,&Stream::put_uvlint,&Stream::get_uvlint      ))
    {
      printf("\ntest failed.\n");
    }


  printf("\n");
}


int
main(int  argc, char**  argv)
{
  Storage  st;

  const char*  path = "test.db";

  auto  f = fopen(path,"rb");

    if(!f)
    {
      test_vlint();

      f = fopen(path,"wb");

      FileStream  fs(f);

      auto&  table = st.new_table("TestTable",TableDef({Header("Number"),Header("String",ValueKind::string),Header("Bool",ValueKind::bool_)}));

      auto&  rec = table.new_record();

      rec.values_[0] = Value(876543210);
      rec.values_[1] = Value("aeiou");
      rec.values_[2] = Value(true);

      st.print();

      st.write(fs);
    }

  else
    {
      FileStream  fs(f);

      st.read(fs);

      st.print();
    }


  return 0;
}




